@@ -14,17 +14,15 @@ use std::{
 };
 
 use libseccomp::ScmpNotifResp;
-use nix::{
-    errno::Errno,
-    fcntl::{OFlag, ResolveFlag},
-};
+use nix::{errno::Errno, fcntl::OFlag};
 use serde::{Serialize, Serializer};
 
 use crate::{
+    compat::ResolveFlag,
     config::{PAGE_SIZE, PROC_FILE},
     elf::ExecutableFile,
     error,
-    fs::fd_status_flags,
+    fd::fd_status_flags,
     kernel::sandbox_path,
     lookup::{safe_open_msym, CanonicalPath},
     path::XPathBuf,
