@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/kernel/mod.rs: Secure computing hooks
 //
-// Copyright (c) 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -406,8 +406,8 @@ where
 
                 if sandbox.is_chroot() {
                     return if caps.contains(Capability::CAP_CHDIR) && path.abs().is_rootfs() {
-                        // SAFETY: Allow `cd /` after chroot.
-                        Ok(unsafe { request.continue_syscall() })
+                        // No-op `cd /` after chroot.
+                        Ok(request.return_syscall(0))
                     } else {
                         // arg.path.is_some() -> is_fd = false: Return ENOENT.
                         Err(Errno::ENOENT)
@@ -419,11 +419,6 @@ where
                 // Validate FD argument.
                 let dirfd = to_valid_fd(req.data.args[arg_idx])?;
 
-                if sandbox.is_chroot() {
-                    // Return EACCES for FD-only system calls such as fchdir.
-                    return Err(if is_fd { Errno::EACCES } else { Errno::ENOENT });
-                }
-
                 if dirfd != libc::AT_FDCWD {
                     // SAFETY: Get the file descriptor before access check
                     // as it may change after which is a TOCTOU vector.
