@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/kernel/ptrace/event/exec.rs: ptrace(2) exec event handler
 //
-// Copyright (c) 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -13,10 +13,9 @@ use std::{
     sync::{Arc, RwLock},
 };
 
-use data_encoding::HEXLOWER;
 use nix::{
     errno::Errno,
-    fcntl::{OFlag, ResolveFlag},
+    fcntl::OFlag,
     sys::{
         ptrace,
         signal::{kill, Signal},
@@ -26,16 +25,15 @@ use nix::{
 
 use crate::{
     cache::ExecResult,
-    compat::{fstatfs64, fstatx, FileStatx, STATX_INO},
+    compat::{fstatx, FileStatx, FsType, ResolveFlag, STATX_INO},
     debug,
     elf::{ElfError, ElfFileType, ElfType, ExecutableFile, LinkingType},
     err::err2no,
     error,
-    fs::AT_BADFD,
+    fd::AT_BADFD,
     log_enabled,
     lookup::safe_open,
     proc::{proc_executables, proc_set_at_secure, SydExecMap},
-    retry::retry_on_eintr,
     sandbox::{Action, Capability, IntegrityError, Sandbox, SandboxGuard},
     syslog::LogLevel,
     warn,
@@ -114,15 +112,15 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                 // Check the comment on has_broken_device_ids() function
                 // for more information.
                 // Assume true on errors for safety.
-                let dev_check = match retry_on_eintr(|| fstatfs64(&fd)) {
-                    Ok(statfs) => !statfs.has_broken_device_ids(),
+                let dev_check = match FsType::get(&fd).map(|fs_type| !fs_type.has_broken_devid()) {
+                    Ok(dev_check) => dev_check,
                     Err(Errno::ENOSYS) => {
                         // Filesystem type does not support this call.
                         // Assume true for safety.
                         true
                     }
                     Err(errno) => {
-                        error!("ctx": "open_elf",
+                        error!("ctx": "exec", "op": "open_elf",
                             "msg": format!("statfs error: {errno}"),
                             "err": errno as i32,
                             "pid": pid.as_raw(), "path": path);
@@ -133,7 +131,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                 let statx = match fstatx(&fd, STATX_INO) {
                     Ok(stat) => stat,
                     Err(errno) => {
-                        error!("ctx": "open_elf",
+                        error!("ctx": "exec", "op": "open_elf",
                             "msg": format!("statx error: {errno}"),
                             "err": errno as i32,
                             "pid": pid.as_raw(), "path": path);
@@ -159,7 +157,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                         dev_minor,
                         bin.inode
                     );
-                    error!("ctx": "open_elf",
+                    error!("ctx": "exec", "op": "open_elf",
                         "msg": error,
                         "pid": pid.as_raw(),"path": path);
                     let _ = kill(pid, Some(Signal::SIGKILL));
@@ -168,7 +166,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                 files.push(File::from(fd));
             }
             Err(errno) => {
-                error!("ctx": "open_elf",
+                error!("ctx": "exec", "op": "open_elf",
                     "msg": format!("open error: {errno}"),
                     "err": errno as i32,
                     "pid": pid.as_raw(), "path": path);
@@ -185,7 +183,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         // Check SegvGuard.
         if let Some(action) = my_sandbox.check_segvguard(path) {
             if action != Action::Filter {
-                error!("ctx": "segvguard",
+                error!("ctx": "exec", "op": "segvguard",
                     "msg": format!("Max crashes {} exceeded, kill process {}",
                         my_sandbox.segvguard_maxcrashes,
                         pid.as_raw()),
@@ -244,7 +242,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                 let (action, msg) = my_sandbox.check_tpe(file, path);
                 if !matches!(action, Action::Allow | Action::Filter) {
                     let msg = msg.as_deref().unwrap_or("?");
-                    error!("ctx": "trusted_path_execution", "err": libc::EACCES,
+                    error!("ctx": "exec", "op": "trusted_path_execution", "err": libc::EACCES,
                         "pid": pid.as_raw(), "sys": "exec", "path": path, "act": action,
                         "msg": format!("exec from untrusted path blocked: {msg}"),
                         "tip": "move the binary to a safe location or use `sandbox/tpe:off'");
@@ -314,7 +312,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
             Err(ElfError::IoError(err)) => {
                 deny_action = Some(Action::Kill);
                 if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                    error!("ctx": "parse_elf",
+                    error!("ctx": "exec", "op": "parse_elf",
                         "msg": format!("io error: {}", err2no(&err)),
                         "err": err2no(&err) as i32,
                         "pid": pid.as_raw(), "path": path);
@@ -323,7 +321,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
             Err(ElfError::BadMagic) => {
                 deny_action = Some(Action::Kill);
                 if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                    error!("ctx": "parse_elf",
+                    error!("ctx": "exec", "op": "parse_elf",
                         "msg": format!("BUG: not an ELF"),
                         "pid": pid.as_raw(), "path": path);
                 }
@@ -331,7 +329,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
             Err(ElfError::Malformed) => {
                 deny_action = Some(Action::Kill);
                 if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                    error!("ctx": "parse_elf",
+                    error!("ctx": "exec", "op": "parse_elf",
                         "msg": format!("BUG: malformed ELF"),
                         "pid": pid.as_raw(), "path": path);
                 }
@@ -349,7 +347,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         {
             deny_action = Some(Action::Kill);
             if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                error!("ctx": "check_elf",
+                error!("ctx": "exec", "op": "check_elf",
                     "msg": "ld.so(8) exec-indirection prevented",
                     "pid": pid.as_raw(), "path": path,
                     "tip": "configure `trace/allow_unsafe_exec_ldso:1'",
@@ -363,7 +361,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         {
             deny_action = Some(Action::Kill);
             if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                error!("ctx": "check_elf",
+                error!("ctx": "exec", "op": "check_elf",
                     "msg": "ELF is not a Position Independent Executable (PIE)",
                     "pid": pid.as_raw(), "path": path,
                     "tip": "configure `trace/allow_unsafe_exec_nopie:1'",
@@ -377,7 +375,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         {
             deny_action = Some(Action::Kill);
             if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                error!("ctx": "check_elf",
+                error!("ctx": "exec", "op": "check_elf",
                     "msg": "ELF has Executable Stack (PT_GNU_STACK)",
                     "pid": pid.as_raw(), "path": path,
                     "tip": "configure `trace/allow_unsafe_exec_stack:1'",
@@ -397,7 +395,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         {
             deny_action = Some(Action::Kill);
             if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                error!("ctx": "check_elf",
+                error!("ctx": "exec", "op": "check_elf",
                     "msg": "32-bit execution prevented",
                     "pid": pid.as_raw(), "path": path,
                     "tip": "configure `trace/deny_exec_elf32:0'",
@@ -417,7 +415,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         {
             deny_action = Some(Action::Kill);
             if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                error!("ctx": "check_elf",
+                error!("ctx": "exec", "op": "check_elf",
                     "msg": "dynamic-link execution prevented",
                     "pid": pid.as_raw(), "path": path,
                     "tip": "configure `trace/deny_exec_elf_dynamic:0'",
@@ -437,7 +435,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         {
             deny_action = Some(Action::Kill);
             if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                error!("ctx": "check_elf",
+                error!("ctx": "exec", "op": "check_elf",
                     "msg": "static-link execution prevented",
                     "pid": pid.as_raw(), "path": path,
                     "tip": "configure `trace/deny_exec_elf_static:0'",
@@ -468,24 +466,24 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                 match result {
                     Ok(Action::Allow) => {}
                     Ok(Action::Warn) => {
-                        warn!("ctx": "verify_elf", "act": Action::Warn,
+                        warn!("ctx": "exec", "op": "verify_elf", "act": Action::Warn,
                             "pid": pid.as_raw(), "path": path,
                             "tip": format!("configure `force+{path}:<checksum>'"));
                     }
                     Ok(Action::Stop) => {
                         deny_action = Some(Action::Stop);
-                        warn!("ctx": "verify_elf", "act": Action::Stop,
+                        warn!("ctx": "exec", "op": "verify_elf", "act": Action::Stop,
                             "pid": pid.as_raw(), "path": path,
                             "tip": format!("configure `force+{path}:<checksum>'"));
                     }
                     Ok(Action::Abort) => {
                         deny_action = Some(Action::Abort);
-                        warn!("ctx": "verify_elf", "act": Action::Abort,
+                        warn!("ctx": "exec", "op": "verify_elf", "act": Action::Abort,
                             "pid": pid.as_raw(), "path": path,
                             "tip": format!("configure `force+{path}:<checksum>'"));
                     }
                     Ok(Action::Exit) => {
-                        error!("ctx": "verify_elf", "act": Action::Exit,
+                        error!("ctx": "exec", "op": "verify_elf", "act": Action::Exit,
                             "pid": pid.as_raw(), "path": path,
                             "tip": format!("configure `force+{path}:<checksum>'"));
                         std::process::exit(libc::EACCES);
@@ -499,14 +497,14 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                             action = Action::Kill;
                         }
                         if action != Action::Filter {
-                            warn!("ctx": "verify_elf", "act": action,
+                            warn!("ctx": "exec", "op": "verify_elf", "act": action,
                                 "pid": pid.as_raw(), "path": path,
                                 "tip": format!("configure `force+{path}:<checksum>'"));
                         }
                     }
                     Err(IntegrityError::Sys(errno)) => {
                         deny_action = Some(Action::Kill);
-                        error!("ctx": "verify_elf",
+                        error!("ctx": "exec", "op": "verify_elf",
                             "msg": format!("system error during ELF checksum calculation: {errno}"),
                             "err": errno as i32,
                             "pid": pid.as_raw(), "path": path,
@@ -523,7 +521,7 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
                             action = Action::Kill;
                         }
                         if !matches!(action, Action::Allow | Action::Filter) {
-                            error!("ctx": "verify_elf", "act": action,
+                            error!("ctx": "exec", "op": "verify_elf", "act": action,
                                 "msg": format!("ELF checksum mismatch: {found} is not {expected}"),
                                 "pid": pid.as_raw(), "path": path,
                                 "tip": format!("configure `force+{path}:<checksum>'"));
@@ -554,12 +552,13 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
         // SAFETY:
         // 1. Sets AT_SECURE.
         // 2. Verifies AT_{E,}{U,G}ID matches Syd's own.
-        match proc_set_at_secure(pid, elf_type) {
+        // 3. Overwrite AT_SYSINFO{,_EHDR} if trace/deny_vdso:1.
+        match proc_set_at_secure(pid, elf_type, my_sandbox.flags.deny_vdso()) {
             Ok(_) | Err(Errno::ESRCH) => {}
             Err(errno) => {
                 deny_action = Some(Action::Kill);
                 if !my_sandbox.filter_path(Capability::CAP_EXEC, path) {
-                    error!("ctx": "secure_exec",
+                    error!("ctx": "exec", "op": "secure_exec",
                         "msg": format!("error setting AT_SECURE: {errno}"),
                         "err": errno as i32,
                         "tip": "configure `trace/allow_unsafe_exec_libc:1'",
@@ -586,24 +585,14 @@ pub(crate) fn sysevent_exec(pid: Pid, cache: &Arc<WorkerCache>, sandbox: &Arc<Rw
             ),
         );
     } else {
-        let _ = ptrace::cont(pid, None);
-
         if log_enabled!(LogLevel::Debug) {
-            let ip_mem = rx.ip_mem.map(|ip_mem| HEXLOWER.encode(&ip_mem));
-            let sp_mem = rx.sp_mem.map(|sp_mem| HEXLOWER.encode(&sp_mem));
-
+            let exe = exe.to_string();
             debug!("ctx": "exec", "op": "verify_exec",
                 "msg": format!("execution of `{path}' of type {exe} approved"),
-                "pid": pid.as_raw(),
-                "path": &path,
-                "exe": &exe.to_string(),
-                "args": rx.args,
-                "ip": rx.ip,
-                "sp": rx.sp,
-                "ip_mem": ip_mem,
-                "sp_mem": sp_mem,
-                "memmap": rx.memmap);
+                "pid": pid.as_raw(), "path": &path, "exe": &exe);
         }
+
+        let _ = ptrace::cont(pid, None);
     }
 }
 
