@@ -13,8 +13,8 @@ use nix::{errno::Errno, fcntl::AtFlags, sys::time::TimeSpec};
 
 use crate::{
     confine::scmp_arch_bits,
-    fs::FsFlags,
     kernel::{syscall_path_handler, to_atflags},
+    lookup::FsFlags,
     req::{PathArgs, SysArg, SysFlags, UNotifyEventRequest},
 };
 
