@@ -142,6 +142,8 @@ pub mod elf;
 pub mod err;
 /// File descriptor utilities
 pub mod fd;
+/// File descriptor map for path canonicalization
+pub(crate) mod filemap;
 /// Filesystem utilities
 pub mod fs;
 /// Utilities for hashing
