@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/proc.rs: /proc utilities
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Based in part upon procps-ng's library/devname.c which is:
 //   Copyright © 2011-2023 Jim Warner <james.warner@comcast.net>
 //   Copyright © 2017-2023 Craig Small <csmall@dropbear.xyz>
@@ -66,7 +66,7 @@ use crate::{
     compat::{getdents64, openat2, statx, ResolveFlag, STATX_INO},
     config::*,
     confine::SydMemoryMap,
-    cookie::{PROCMAP_QUERY_COOKIE_ARG3, PROCMAP_QUERY_COOKIE_ARG4, PROCMAP_QUERY_COOKIE_ARG5},
+    cookie::{CookieIdx, SYSCOOKIE_POOL},
     elf::ElfType,
     err::{err2no, proc_error_to_errno},
     fd::{is_empty_file, is_open_fd, parse_fd, AT_BADFD},
@@ -353,9 +353,9 @@ pub fn procmap_query<Fd: AsFd>(
             fd.as_fd().as_raw_fd(),
             PROCMAP_QUERY,
             &mut q,
-            *PROCMAP_QUERY_COOKIE_ARG3,
-            *PROCMAP_QUERY_COOKIE_ARG4,
-            *PROCMAP_QUERY_COOKIE_ARG5,
+            SYSCOOKIE_POOL.get(CookieIdx::ProcmapQueryArg3),
+            SYSCOOKIE_POOL.get(CookieIdx::ProcmapQueryArg4),
+            SYSCOOKIE_POOL.get(CookieIdx::ProcmapQueryArg5),
         )
     })
     .map(|_| q)
@@ -1706,11 +1706,12 @@ pub fn proc_unix_inodes(pid: Pid) -> Result<SydHashSet<u64>, Errno> {
 ///
 /// * `pid` - The PID of the target process.
 /// * `elf_type` - The ELF type (Elf32 or Elf64) of the target process.
+/// * `deny_vdso` - True to deny access to vDSO by overwriting AT_SYSINFO{,_EHDR}.
 ///
 /// # Returns
 ///
 /// Returns `Ok(())` on success, or an `Err(Errno)` if an error occurs.
-pub fn proc_set_at_secure(pid: Pid, elf_type: ElfType) -> Result<(), Errno> {
+pub fn proc_set_at_secure(pid: Pid, elf_type: ElfType, deny_vdso: bool) -> Result<(), Errno> {
     // This function requires CONFIG_CROSS_MEMORY_ATTACH,
     // unless Syd is explicitly configured to fallback.
     let mut proc_mem = if Sandbox::use_proc_pid_mem() {
@@ -1881,6 +1882,14 @@ pub fn proc_set_at_secure(pid: Pid, elf_type: ElfType) -> Result<(), Errno> {
     const AT_REQKEY: &[usize] = &[AT_UID, AT_EUID, AT_GID, AT_EGID, AT_SECURE];
     let mut required_index = 0;
 
+    // Used to deny access to vDSO as necessary.
+    //
+    // 1. libc crate does not define these for all arches we support,
+    //    however the values are consistent across all.
+    // 2. AT_SECURE comes after AT_SYSINFO{,_EHDR} in the auxiliary vector.
+    const AT_SYSINFO: usize = 32;
+    const AT_SYSINFO_EHDR: usize = 33;
+
     // Store the UID/GID values for comparison.
     // We're going to check these values against ours
     // and fail as necessary.
@@ -1968,6 +1977,28 @@ pub fn proc_set_at_secure(pid: Pid, elf_type: ElfType) -> Result<(), Errno> {
         } else if required_index > 0 {
             // An unexpected key appeared; validation fails!
             return Err(Errno::EACCES);
+        } else if deny_vdso && (key == AT_SYSINFO || key == AT_SYSINFO_EHDR) {
+            // Deny access to vDSO as requested.
+            // AT_SYS_INFO{,_EHDR} always comes before AT_SECURE.
+            //
+            // Overwrite the key and value in the local buffer:
+            // Overwrite the key with AT_SECURE.
+            // Overwrite the value with 1.
+            let key = usize_to_ne_bytes(AT_SECURE, sizeof_ptr);
+            let val = usize_to_ne_bytes(1, sizeof_ptr);
+            buf[offset..key_end].copy_from_slice(&key);
+            buf[key_end..val_end].copy_from_slice(&val);
+
+            // Write the modified value back to the target process.
+            if proc_write(
+                proc_mem.as_mut(),
+                pid,
+                &buf[offset..val_end],
+                sp.checked_add(offset).ok_or(Errno::EINVAL)?,
+            )? != sizeof_ptr2
+            {
+                return Err(Errno::EIO);
+            }
         } else if key == AT_NULL {
             break;
         }
