@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/req.rs: seccomp(2) notify request handling
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Based in part upon greenhook which is under public domain.
 //
 // SPDX-License-Identifier: GPL-3.0
@@ -51,12 +51,12 @@ use crate::{
     confine::{is_valid_ptr, op2name, scmp_arch_bits, ScmpNotifReq, SydArch, Sydcall, EIDRM},
     error,
     fd::{fd_status_flags, pidfd_getfd, pidfd_open, pidfd_send_signal, to_valid_fd, PIDFD_THREAD},
-    fs::{process_mrelease, seccomp_notify_addfd, seccomp_notify_id_valid, unix_inodes},
+    fs::{process_mrelease, seccomp_notify_addfd, seccomp_notify_id_valid},
     lookup::{file_type, safe_canonicalize, safe_open_msym, CanonicalPath, FileType, FsFlags},
     path::{XPath, XPathBuf, PATH_MAX, PATH_MIN},
     proc::{
         proc_auxv, proc_comm, proc_get_vma, proc_rand_fd, proc_stack_pointer, proc_status,
-        proc_tgid, proc_unix_inodes,
+        proc_tgid,
     },
     sandbox::{Action, Flags, Sandbox, SandboxGuard},
     workers::WorkerCache,
@@ -238,7 +238,6 @@ impl RemoteProcess {
     /// and fd transfers. Otherwise, the caller must validate
     /// to verify the path read from sandbox process memory
     /// is what's expected.
-    #[expect(clippy::cognitive_complexity)]
     #[expect(clippy::type_complexity)]
     pub(crate) fn read_path(
         &self,
@@ -1074,7 +1073,6 @@ impl UNotifyEventRequest {
     ///
     /// Used for informational purposes at recvfrom(2) boundary.
     pub(crate) fn find_unix_addr(&self, base: &XPath) -> Result<UnixAddr, Errno> {
-        self.clr_unix()?; // cleanup bind-map from unused inodes.
         let unix_map = self
             .cache
             .unix_map
@@ -1157,30 +1155,7 @@ impl UNotifyEventRequest {
                 });
             }
         }
-        let unix_len = unix_map.len();
 
-        // SAFETY: Do _not_ hold a write lock during /proc read.
-        drop(unix_map);
-
-        // Cleanup unix map from unused inodes as necessary.
-        if unix_len > 128 {
-            self.clr_unix()?;
-        }
-
-        Ok(())
-    }
-
-    // Cleanup unix map from unused inodes.
-    pub(crate) fn clr_unix(&self) -> Result<(), Errno> {
-        // Try netlink(7) first, fallback to proc_net(5).
-        // netlink(7) requires CONFIG_UNIX_DIAG enabled.
-        let inodes = unix_inodes().or_else(|_| proc_unix_inodes(self.scmpreq.pid()))?;
-        let mut unix_map = self
-            .cache
-            .unix_map
-            .write()
-            .unwrap_or_else(|err| err.into_inner());
-        unix_map.retain(|inode, _| inodes.contains(inode));
         Ok(())
     }
 
