@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/workers/aes.rs: `syd_aes' encryption thread
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -31,19 +31,14 @@ use nix::{
 use serde::{Serialize, Serializer};
 
 #[cfg(target_arch = "x86")]
-use crate::cookie::FTRUNCATE64_COOKIE_ARG3;
+use crate::cookie::CookieIdx::Ftruncate64Arg3;
 use crate::{
     config::*,
     confine::{
         confine_scmp_clone, confine_scmp_clone3, confine_scmp_fadvise, confine_scmp_madvise,
         confine_scmp_write, scmp_add_setid_rules, ExportMode,
     },
-    cookie::{
-        safe_ftruncate64, safe_pipe2, ACCEPT4_COOKIE_ARG4, ACCEPT4_COOKIE_ARG5,
-        FTRUNCATE64_COOKIE_ARG4, FTRUNCATE64_COOKIE_ARG5, FTRUNCATE_COOKIE_ARG2,
-        FTRUNCATE_COOKIE_ARG3, FTRUNCATE_COOKIE_ARG4, FTRUNCATE_COOKIE_ARG5, PIPE2_COOKIE_ARG2,
-        PIPE2_COOKIE_ARG3, PIPE2_COOKIE_ARG4, PIPE2_COOKIE_ARG5, SYS_ACCEPT4,
-    },
+    cookie::{safe_ftruncate64, safe_pipe2, CookieIdx, SYSCOOKIE_POOL, SYS_ACCEPT4},
     debug,
     err::{err2no, SydJoinHandle, SydResult},
     error,
@@ -271,10 +266,10 @@ impl AesWorker {
                 let mut rules = vec![];
                 if !flags.allow_unsafe_nocookie() {
                     rules.extend(&[
-                        scmp_cmp!($arg2 == (*FTRUNCATE_COOKIE_ARG2).into()),
-                        scmp_cmp!($arg3 == (*FTRUNCATE_COOKIE_ARG3).into()),
-                        scmp_cmp!($arg4 == (*FTRUNCATE_COOKIE_ARG4).into()),
-                        scmp_cmp!($arg5 == (*FTRUNCATE_COOKIE_ARG5).into()),
+                        scmp_cmp!($arg2 == SYSCOOKIE_POOL.get(CookieIdx::FtruncateArg2).into()),
+                        scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::FtruncateArg3).into()),
+                        scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::FtruncateArg4).into()),
+                        scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::FtruncateArg5).into()),
                     ]);
                 }
 
@@ -299,9 +294,9 @@ impl AesWorker {
                 if !flags.allow_unsafe_nocookie() {
                     rules.extend(&[
                         #[cfg(target_arch = "x86")]
-                        scmp_cmp!($arg3 == (*FTRUNCATE64_COOKIE_ARG3).into()),
-                        scmp_cmp!($arg4 == (*FTRUNCATE64_COOKIE_ARG4).into()),
-                        scmp_cmp!($arg5 == (*FTRUNCATE64_COOKIE_ARG5).into()),
+                        scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(Ftruncate64Arg3).into()),
+                        scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::Ftruncate64Arg4).into()),
+                        scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Ftruncate64Arg5).into()),
                     ]);
                 }
 
@@ -328,8 +323,8 @@ impl AesWorker {
             let mut rules = vec![];
             if !flags.allow_unsafe_nocookie() {
                 rules.extend(&[
-                    scmp_cmp!($arg4 == (*ACCEPT4_COOKIE_ARG4).into()),
-                    scmp_cmp!($arg5 == (*ACCEPT4_COOKIE_ARG5).into()),
+                    scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::Accept4Arg4).into()),
+                    scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Accept4Arg5).into()),
                 ]);
             }
 
@@ -360,10 +355,10 @@ impl AesWorker {
             let mut rules = vec![scmp_cmp!($arg1 == OFlag::O_CLOEXEC.bits() as u64)];
             if !flags.allow_unsafe_nocookie() {
                 rules.extend(&[
-                    scmp_cmp!($arg2 == (*PIPE2_COOKIE_ARG2).into()),
-                    scmp_cmp!($arg3 == (*PIPE2_COOKIE_ARG3).into()),
-                    scmp_cmp!($arg4 == (*PIPE2_COOKIE_ARG4).into()),
-                    scmp_cmp!($arg5 == (*PIPE2_COOKIE_ARG5).into()),
+                    scmp_cmp!($arg2 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg2).into()),
+                    scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg3).into()),
+                    scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg4).into()),
+                    scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg5).into()),
                 ]);
             }
 
@@ -472,7 +467,6 @@ impl AesWorker {
         Ok(ctx)
     }
 
-    #[expect(clippy::cognitive_complexity)]
     pub(crate) fn try_spawn(self) -> Result<SydJoinHandle<()>, Errno> {
         thread::Builder::new()
             .name("syd_aes".to_string())
