@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/workers/emu.rs: `syd_emu' emulator threads
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Based in part upon rusty_pool which is:
 //     Copyright (c) Robin Friedli <robinfriedli@icloud.com>
 //     SPDX-License-Identifier: Apache-2.0
@@ -32,10 +32,6 @@ use nix::{
     unistd::{Gid, Pid, Uid},
 };
 
-#[cfg(target_arch = "x86")]
-use crate::cookie::FTRUNCATE64_COOKIE_ARG3;
-#[cfg(target_arch = "x86")]
-use crate::cookie::TRUNCATE64_COOKIE_ARG3;
 use crate::{
     alert,
     compat::{seccomp_notif, seccomp_notif_resp},
@@ -46,26 +42,7 @@ use crate::{
         X32_SYSCALL_BIT,
     },
     cookie::{
-        ACCEPT4_COOKIE_ARG4, ACCEPT4_COOKIE_ARG5, BIND_COOKIE_ARG3, BIND_COOKIE_ARG4,
-        BIND_COOKIE_ARG5, CONNECT_COOKIE_ARG3, CONNECT_COOKIE_ARG4, CONNECT_COOKIE_ARG5,
-        FCHDIR_COOKIE_ARG1, FCHDIR_COOKIE_ARG2, FCHDIR_COOKIE_ARG3, FCHDIR_COOKIE_ARG4,
-        FCHDIR_COOKIE_ARG5, FTRUNCATE64_COOKIE_ARG4, FTRUNCATE64_COOKIE_ARG5,
-        FTRUNCATE_COOKIE_ARG2, FTRUNCATE_COOKIE_ARG3, FTRUNCATE_COOKIE_ARG4, FTRUNCATE_COOKIE_ARG5,
-        LINKAT_COOKIE_ARG5, MEMFD_CREATE_COOKIE_ARG2, MEMFD_CREATE_COOKIE_ARG3,
-        MEMFD_CREATE_COOKIE_ARG4, MEMFD_CREATE_COOKIE_ARG5, OPENAT2_COOKIE_ARG4,
-        OPENAT2_COOKIE_ARG5, PIPE2_COOKIE_ARG2, PIPE2_COOKIE_ARG3, PIPE2_COOKIE_ARG4,
-        PIPE2_COOKIE_ARG5, PROCMAP_QUERY_COOKIE_ARG3, PROCMAP_QUERY_COOKIE_ARG4,
-        PROCMAP_QUERY_COOKIE_ARG5, RENAMEAT2_COOKIE_ARG5, SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG3,
-        SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG4, SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG5,
-        SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG3, SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG4,
-        SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG5, SOCKETPAIR_COOKIE_ARG4, SOCKETPAIR_COOKIE_ARG5,
-        SOCKET_COOKIE_ARG3, SOCKET_COOKIE_ARG4, SOCKET_COOKIE_ARG5, SYS_ACCEPT4, SYS_BIND,
-        SYS_CONNECT, SYS_SOCKET, SYS_SOCKETPAIR, TRUNCATE64_COOKIE_ARG4, TRUNCATE64_COOKIE_ARG5,
-        TRUNCATE_COOKIE_ARG2, TRUNCATE_COOKIE_ARG3, TRUNCATE_COOKIE_ARG4, TRUNCATE_COOKIE_ARG5,
-        UMASK_COOKIE_ARG1, UMASK_COOKIE_ARG2, UMASK_COOKIE_ARG3, UMASK_COOKIE_ARG4,
-        UMASK_COOKIE_ARG5, UNAME_COOKIE_ARG1, UNAME_COOKIE_ARG2, UNAME_COOKIE_ARG3,
-        UNAME_COOKIE_ARG4, UNAME_COOKIE_ARG5, UNLINKAT_COOKIE_ARG3, UNLINKAT_COOKIE_ARG4,
-        UNLINKAT_COOKIE_ARG5,
+        CookieIdx, SYSCOOKIE_POOL, SYS_ACCEPT4, SYS_BIND, SYS_CONNECT, SYS_SOCKET, SYS_SOCKETPAIR,
     },
     err::{err2no, SydJoinHandle, SydResult},
     fd::{close, AT_EXECVE_CHECK},
@@ -485,9 +462,9 @@ impl Worker {
                     if !flags.allow_unsafe_nocookie() && *ioctl_request == SECCOMP_IOCTL_NOTIF_ADDFD
                     {
                         rules.extend(&[
-                            scmp_cmp!($arg3 == (*SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG3).into()),
-                            scmp_cmp!($arg4 == (*SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG4).into()),
-                            scmp_cmp!($arg5 == (*SECCOMP_IOCTL_NOTIF_ADDFD_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::SeccompIoctlNotifAddfdArg3).into()),
+                            scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::SeccompIoctlNotifAddfdArg4).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::SeccompIoctlNotifAddfdArg5).into()),
                         ]);
                     }
 
@@ -500,9 +477,9 @@ impl Worker {
                     if !flags.allow_unsafe_nocookie() && *ioctl_request == SECCOMP_IOCTL_NOTIF_SEND
                     {
                         rules.extend(&[
-                            scmp_cmp!($arg3 == (*SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG3).into()),
-                            scmp_cmp!($arg4 == (*SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG4).into()),
-                            scmp_cmp!($arg5 == (*SECCOMP_IOCTL_NOTIF_SEND_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::SeccompIoctlNotifSendArg3).into()),
+                            scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::SeccompIoctlNotifSendArg4).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::SeccompIoctlNotifSendArg5).into()),
                         ]);
                     }
 
@@ -523,9 +500,9 @@ impl Worker {
                         Vec::with_capacity(if flags.allow_unsafe_nocookie() { 1 } else { 4 });
                     if !flags.allow_unsafe_nocookie() {
                         rules.extend(&[
-                            scmp_cmp!($arg3 == (*PROCMAP_QUERY_COOKIE_ARG3).into()),
-                            scmp_cmp!($arg4 == (*PROCMAP_QUERY_COOKIE_ARG4).into()),
-                            scmp_cmp!($arg5 == (*PROCMAP_QUERY_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::ProcmapQueryArg3).into()),
+                            scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::ProcmapQueryArg4).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::ProcmapQueryArg5).into()),
                         ]);
                     }
 
@@ -572,10 +549,10 @@ impl Worker {
                 let mut rules = vec![scmp_cmp!($arg1 == OFlag::O_CLOEXEC.bits() as u64)];
                 if !flags.allow_unsafe_nocookie() {
                     rules.extend(&[
-                        scmp_cmp!($arg2 == (*PIPE2_COOKIE_ARG2).into()),
-                        scmp_cmp!($arg3 == (*PIPE2_COOKIE_ARG3).into()),
-                        scmp_cmp!($arg4 == (*PIPE2_COOKIE_ARG4).into()),
-                        scmp_cmp!($arg5 == (*PIPE2_COOKIE_ARG5).into()),
+                        scmp_cmp!($arg2 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg2).into()),
+                        scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg3).into()),
+                        scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg4).into()),
+                        scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg5).into()),
                     ]);
                 }
 
@@ -662,10 +639,10 @@ impl Worker {
                         ScmpAction::Allow,
                         syscall,
                         &[
-                            scmp_cmp!($arg2 == (*MEMFD_CREATE_COOKIE_ARG2).into()),
-                            scmp_cmp!($arg3 == (*MEMFD_CREATE_COOKIE_ARG3).into()),
-                            scmp_cmp!($arg4 == (*MEMFD_CREATE_COOKIE_ARG4).into()),
-                            scmp_cmp!($arg5 == (*MEMFD_CREATE_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg2 == SYSCOOKIE_POOL.get(CookieIdx::MemfdCreateArg2).into()),
+                            scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::MemfdCreateArg3).into()),
+                            scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::MemfdCreateArg4).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::MemfdCreateArg5).into()),
                         ],
                     )?;
                 }
@@ -686,7 +663,7 @@ impl Worker {
                         syscall,
                         &[
                             scmp_cmp!($arg0 <= RawFd::MAX as u64),
-                            scmp_cmp!($arg5 == (*RENAMEAT2_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Renameat2Arg5).into()),
                         ],
                     )?;
                 }
@@ -705,10 +682,10 @@ impl Worker {
                         ScmpAction::Allow,
                         syscall,
                         &[
-                            scmp_cmp!($arg2 == (*TRUNCATE_COOKIE_ARG2).into()),
-                            scmp_cmp!($arg3 == (*TRUNCATE_COOKIE_ARG3).into()),
-                            scmp_cmp!($arg4 == (*TRUNCATE_COOKIE_ARG4).into()),
-                            scmp_cmp!($arg5 == (*TRUNCATE_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg2 == SYSCOOKIE_POOL.get(CookieIdx::TruncateArg2).into()),
+                            scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::TruncateArg3).into()),
+                            scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::TruncateArg4).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::TruncateArg5).into()),
                         ],
                     )?;
                 }
@@ -727,9 +704,9 @@ impl Worker {
                         syscall,
                         &[
                             #[cfg(target_arch = "x86")]
-                            scmp_cmp!($arg3 == (*TRUNCATE64_COOKIE_ARG3).into()),
-                            scmp_cmp!($arg4 == (*TRUNCATE64_COOKIE_ARG4).into()),
-                            scmp_cmp!($arg5 == (*TRUNCATE64_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::Truncate64Arg3).into()),
+                            scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::Truncate64Arg4).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Truncate64Arg5).into()),
                         ],
                     )?;
                 }
@@ -747,10 +724,10 @@ impl Worker {
                         ScmpAction::Allow,
                         syscall,
                         &[
-                            scmp_cmp!($arg2 == (*FTRUNCATE_COOKIE_ARG2).into()),
-                            scmp_cmp!($arg3 == (*FTRUNCATE_COOKIE_ARG3).into()),
-                            scmp_cmp!($arg4 == (*FTRUNCATE_COOKIE_ARG4).into()),
-                            scmp_cmp!($arg5 == (*FTRUNCATE_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg2 == SYSCOOKIE_POOL.get(CookieIdx::FtruncateArg2).into()),
+                            scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::FtruncateArg3).into()),
+                            scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::FtruncateArg4).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::FtruncateArg5).into()),
                         ],
                     )?;
                 }
@@ -769,9 +746,9 @@ impl Worker {
                         syscall,
                         &[
                             #[cfg(target_arch = "x86")]
-                            scmp_cmp!($arg3 == (*FTRUNCATE64_COOKIE_ARG3).into()),
-                            scmp_cmp!($arg4 == (*FTRUNCATE64_COOKIE_ARG4).into()),
-                            scmp_cmp!($arg5 == (*FTRUNCATE64_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::Ftruncate64Arg3).into()),
+                            scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::Ftruncate64Arg4).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Ftruncate64Arg5).into()),
                         ],
                     )?;
                 }
@@ -794,7 +771,7 @@ impl Worker {
                         syscall,
                         &[
                             scmp_cmp!($arg0 == PROC_FD() as u64),
-                            scmp_cmp!($arg5 == (*LINKAT_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::LinkatArg5).into()),
                         ],
                     )?;
                 }
@@ -815,9 +792,9 @@ impl Worker {
                         syscall,
                         &[
                             scmp_cmp!($arg0 <= RawFd::MAX as u64),
-                            scmp_cmp!($arg3 == (*UNLINKAT_COOKIE_ARG3).into()),
-                            scmp_cmp!($arg4 == (*UNLINKAT_COOKIE_ARG4).into()),
-                            scmp_cmp!($arg5 == (*UNLINKAT_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::UnlinkatArg3).into()),
+                            scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::UnlinkatArg4).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::UnlinkatArg5).into()),
                         ],
                     )?;
                 }
@@ -838,8 +815,8 @@ impl Worker {
                         syscall,
                         &[
                             scmp_cmp!($arg0 <= RawFd::MAX as u64),
-                            scmp_cmp!($arg4 == (*OPENAT2_COOKIE_ARG4).into()),
-                            scmp_cmp!($arg5 == (*OPENAT2_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::Openat2Arg4).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Openat2Arg5).into()),
                         ],
                     )?;
                 }
@@ -858,11 +835,11 @@ impl Worker {
                         ScmpAction::Allow,
                         syscall,
                         &[
-                            scmp_cmp!($arg1 == (*UNAME_COOKIE_ARG1).into()),
-                            scmp_cmp!($arg2 == (*UNAME_COOKIE_ARG2).into()),
-                            scmp_cmp!($arg3 == (*UNAME_COOKIE_ARG3).into()),
-                            scmp_cmp!($arg4 == (*UNAME_COOKIE_ARG4).into()),
-                            scmp_cmp!($arg5 == (*UNAME_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg1 == SYSCOOKIE_POOL.get(CookieIdx::UnameArg1).into()),
+                            scmp_cmp!($arg2 == SYSCOOKIE_POOL.get(CookieIdx::UnameArg2).into()),
+                            scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::UnameArg3).into()),
+                            scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::UnameArg4).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::UnameArg5).into()),
                         ],
                     )?;
                 }
@@ -883,11 +860,11 @@ impl Worker {
                         syscall,
                         &[
                             scmp_cmp!($arg0 <= RawFd::MAX as u64),
-                            scmp_cmp!($arg1 == (*FCHDIR_COOKIE_ARG1).into()),
-                            scmp_cmp!($arg2 == (*FCHDIR_COOKIE_ARG2).into()),
-                            scmp_cmp!($arg3 == (*FCHDIR_COOKIE_ARG3).into()),
-                            scmp_cmp!($arg4 == (*FCHDIR_COOKIE_ARG4).into()),
-                            scmp_cmp!($arg5 == (*FCHDIR_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg1 == SYSCOOKIE_POOL.get(CookieIdx::FchdirArg1).into()),
+                            scmp_cmp!($arg2 == SYSCOOKIE_POOL.get(CookieIdx::FchdirArg2).into()),
+                            scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::FchdirArg3).into()),
+                            scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::FchdirArg4).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::FchdirArg5).into()),
                         ],
                     )?;
                 }
@@ -906,11 +883,11 @@ impl Worker {
                         ScmpAction::Allow,
                         syscall,
                         &[
-                            scmp_cmp!($arg1 == (*UMASK_COOKIE_ARG1).into()),
-                            scmp_cmp!($arg2 == (*UMASK_COOKIE_ARG2).into()),
-                            scmp_cmp!($arg3 == (*UMASK_COOKIE_ARG3).into()),
-                            scmp_cmp!($arg4 == (*UMASK_COOKIE_ARG4).into()),
-                            scmp_cmp!($arg5 == (*UMASK_COOKIE_ARG5).into()),
+                            scmp_cmp!($arg1 == SYSCOOKIE_POOL.get(CookieIdx::UmaskArg1).into()),
+                            scmp_cmp!($arg2 == SYSCOOKIE_POOL.get(CookieIdx::UmaskArg2).into()),
+                            scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::UmaskArg3).into()),
+                            scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::UmaskArg4).into()),
+                            scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::UmaskArg5).into()),
                         ],
                     )?;
                 }
@@ -1020,9 +997,9 @@ impl Worker {
             let mut rules = vec![];
             if !flags.allow_unsafe_nocookie() {
                 rules.extend(&[
-                    scmp_cmp!($arg3 == (*SOCKET_COOKIE_ARG3).into()),
-                    scmp_cmp!($arg4 == (*SOCKET_COOKIE_ARG4).into()),
-                    scmp_cmp!($arg5 == (*SOCKET_COOKIE_ARG5).into()),
+                    scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::SocketArg3).into()),
+                    scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::SocketArg4).into()),
+                    scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::SocketArg5).into()),
                 ]);
             }
 
@@ -1087,8 +1064,8 @@ impl Worker {
             let mut rules = vec![];
             if !flags.allow_unsafe_nocookie() {
                 rules.extend(&[
-                    scmp_cmp!($arg4 == (*SOCKETPAIR_COOKIE_ARG4).into()),
-                    scmp_cmp!($arg5 == (*SOCKETPAIR_COOKIE_ARG5).into()),
+                    scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::SocketpairArg4).into()),
+                    scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::SocketpairArg5).into()),
                 ]);
             }
 
@@ -1127,8 +1104,8 @@ impl Worker {
             let mut rules = vec![];
             if !flags.allow_unsafe_nocookie() {
                 rules.extend(&[
-                    scmp_cmp!($arg4 == (*ACCEPT4_COOKIE_ARG4).into()),
-                    scmp_cmp!($arg5 == (*ACCEPT4_COOKIE_ARG5).into()),
+                    scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::Accept4Arg4).into()),
+                    scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Accept4Arg5).into()),
                 ]);
             }
 
@@ -1161,9 +1138,9 @@ impl Worker {
             let mut rules = vec![];
             if !flags.allow_unsafe_nocookie() {
                 rules.extend(&[
-                    scmp_cmp!($arg3 == (*BIND_COOKIE_ARG3).into()),
-                    scmp_cmp!($arg4 == (*BIND_COOKIE_ARG4).into()),
-                    scmp_cmp!($arg5 == (*BIND_COOKIE_ARG5).into()),
+                    scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::BindArg3).into()),
+                    scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::BindArg4).into()),
+                    scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::BindArg5).into()),
                 ]);
             }
 
@@ -1196,9 +1173,9 @@ impl Worker {
             let mut rules = vec![];
             if !flags.allow_unsafe_nocookie() {
                 rules.extend(&[
-                    scmp_cmp!($arg3 == (*CONNECT_COOKIE_ARG3).into()),
-                    scmp_cmp!($arg4 == (*CONNECT_COOKIE_ARG4).into()),
-                    scmp_cmp!($arg5 == (*CONNECT_COOKIE_ARG5).into()),
+                    scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::ConnectArg3).into()),
+                    scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::ConnectArg4).into()),
+                    scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::ConnectArg5).into()),
                 ]);
             }
 
