@@ -29,7 +29,7 @@ use crate::{
     cache::SysInterrupt,
     config::*,
     confine::{confine_scmp_madvise, confine_scmp_write, scmp_add_setid_rules, ExportMode},
-    cookie::{OPENAT2_COOKIE_ARG4, OPENAT2_COOKIE_ARG5},
+    cookie::{CookieIdx, SYSCOOKIE_POOL},
     err::{err2no, scmp2no, SydJoinHandle, SydResult},
     error,
     fd::closeexcept,
@@ -485,8 +485,8 @@ impl Interrupter {
                     syscall,
                     &[
                         scmp_cmp!($arg0 == PROC_FD() as u64),
-                        scmp_cmp!($arg4 == (*OPENAT2_COOKIE_ARG4).into()),
-                        scmp_cmp!($arg5 == (*OPENAT2_COOKIE_ARG5).into()),
+                        scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::Openat2Arg4).into()),
+                        scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Openat2Arg5).into()),
                     ],
                 )?;
             }
